# -*- coding: utf-8 -*-

import sys
import pastel

from contextlib import contextmanager


class PseudoTTY(object):
    def __init__(self, underlying):
        self._underlying = underlying

    def __getattr__(self, name):
        return getattr(self._underlying, name)

    def isatty(self):
        return True


@contextmanager
def mock_stdout():
    original = sys.stdout
    sys.stdout = PseudoTTY(sys.stdout)

    yield

    sys.stdout = original


def test_text():
    with mock_stdout():
        assert "\033[32msome info\033[0m" == pastel.colorize("<info>some info</info>")


def test_colorize():
    with mock_stdout():
        pastel.with_colors(False)
        assert "some info" == pastel.colorize("<info>some info</info>")

        pastel.with_colors(True)
        assert "\033[32msome info\033[0m" == pastel.colorize("<info>some info</info>")


def test_add_remove_style():
    with mock_stdout():
        pastel.add_style("success", "green")

        assert "\033[32msome info\033[0m" == pastel.colorize(
            "<success>some info</success>"
        )

        pastel.remove_style("success")

        assert "<success>some info</success>" == pastel.colorize(
            "<success>some info</success>"
        )


def test_pastel():
    p = pastel.pastel()
    assert isinstance(p, pastel.Pastel)



# -*- coding: utf-8 -*-

import pytest

from pastel import Pastel


def test_empty_tag(pastel):
    assert "foo<>bar" == pastel.colorize("foo<>bar")


def test_lg_char_escaping(pastel):
    assert "foo<bar" == pastel.colorize("foo\\<bar")
    assert "<info>some info</info>" == pastel.colorize("\\<info>some info\\</info>")
    assert "\\<info>some info\\</info>" == pastel.escape("<info>some info</info>")


def test_bundled_styles(pastel):
    assert pastel.has_style("error")
    assert pastel.has_style("info")
    assert pastel.has_style("comment")
    assert pastel.has_style("question")

    assert "\033[97;41msome error\033[0m" == pastel.colorize(
        "<error>some error</error>"
    )
    assert "\033[32msome info\033[0m" == pastel.colorize("<info>some info</info>")
    assert "\033[33msome comment\033[0m" == pastel.colorize(
        "<comment>some comment</comment>"
    )
    assert "\033[30;46msome question\033[0m" == pastel.colorize(
        "<question>some question</question>"
    )


def test_nested_styles(pastel):
    assert (
        "\033[97;41msome \033[0m\033[32msome info\033[0m\033[97;41m error\033[0m"
        == pastel.colorize("<error>some <info>some info</info> error</error>")
    )


def test_adjacent_style(pastel):
    assert "\033[97;41msome error\033[0m\033[32msome info\033[0m" == pastel.colorize(
        "<error>some error</error><info>some info</info>"
    )


def test_style_matching_non_greedy(pastel):
    assert "(\033[32m>=2.0,<2.3\033[0m)" == pastel.colorize("(<info>>=2.0,<2.3</info>)")


def test_style_escaping(pastel):
    assert "(\033[32mz>=2.0,<a2.3\033[0m)" == pastel.colorize(
        "(<info>%s</info>)" % pastel.escape("z>=2.0,<a2.3")
    )


def test_deep_nested_style(pastel):
    assert (
        "\033[97;41merror\033[0m\033[32minfo\033[0m\033[33mcomment\033[0m\033[97;41merror\033[0m"
        == pastel.colorize(
            "<error>error<info>info<comment>comment</comment></info>error</error>"
        )
    )


def test_new_style(pastel):
    pastel.add_style("test", "blue", "white")

    assert pastel.style("test") != pastel.style("info")

    pastel.add_style("b", "blue", "white")

    assert (
        "\033[34;107msome \033[0m\033[34;107mcustom\033[0m\033[34;107m msg\033[0m"
        == pastel.colorize("<test>some <b>custom</b> msg</test>")
    )

    pastel.remove_style("test")
    pastel.remove_style("b")

    assert "<test>some <b>custom</b> msg</test>" == pastel.colorize(
        "<test>some <b>custom</b> msg</test>"
    )

    with pytest.raises(ValueError):
        pastel.remove_style("b")


def test_redefined_style(pastel):
    pastel.add_style("info", "blue", "white")

    assert "\033[34;107msome custom msg\033[0m" == pastel.colorize(
        "<info>some custom msg</info>"
    )


def test_inline_style(pastel):
    assert "\033[34;41msome text\033[0m" == pastel.colorize(
        "<fg=blue;bg=red>some text</>"
    )
    assert "\033[34;41msome text\033[0m" == pastel.colorize(
        "<fg=blue;bg=red>some text</fg=blue;bg=red>"
    )
    assert "\033[34;41;1msome text\033[0m" == pastel.colorize(
        "<fg=blue;bg=red;options=bold>some text</>"
    )


def test_non_style_tag(pastel):
    expected = (
        "\033[32msome \033[0m\033[32m<tag>\033[0m\033[32m \033[0m\033[32m<setting=value>\033[0m\033[32m"
        " styled \033[0m\033[32m<p>\033[0m\033[32msingle-char tag\033[0m\033[32m</p>\033[0m"
    )

    assert expected == pastel.colorize(
        "<info>some <tag> <setting=value> styled <p>single-char tag</p></info>"
    )


def test_non_decorated_pastel(non_decorated_pastel):
    pastel = non_decorated_pastel

    assert pastel.has_style("error")
    assert pastel.has_style("info")
    assert pastel.has_style("comment")
    assert pastel.has_style("question")

    assert "some error" == pastel.colorize("<error>some error</error>")
    assert "some info" == pastel.colorize("<info>some info</info>")
    assert "some comment" == pastel.colorize("<comment>some comment</comment>")
    assert "some question" == pastel.colorize("<question>some question</question>")

    pastel.with_colors(True)

    assert "\033[97;41msome error\033[0m" == pastel.colorize(
        "<error>some error</error>"
    )
    assert "\033[32msome info\033[0m" == pastel.colorize("<info>some info</info>")
    assert "\033[33msome comment\033[0m" == pastel.colorize(
        "<comment>some comment</comment>"
    )
    assert "\033[30;46msome question\033[0m" == pastel.colorize(
        "<question>some question</question>"
    )


@pytest.mark.parametrize(
    "expected, message",
    [
        (
            """\033[32m
some text\033[0m""",
            """<info>
some text</info>""",
        ),
        (
            """\033[32msome text
\033[0m""",
            """<info>some text
</info>""",
        ),
        (
            """\033[32m
some text
\033[0m""",
            """<info>
some text
</info>""",
        ),
        (
            """\033[32m
some text
more text
\033[0m""",
            """<info>
some text
more text
</info>""",
        ),
    ],
)
def test_content_with_line_breaks(pastel, expected, message):
    assert expected == pastel.colorize(message)



# -*- coding: utf-8 -*-

from collections import OrderedDict


class Style(object):

    FOREGROUND_COLORS = {
        "black": 30,
        "red": 31,
        "green": 32,
        "yellow": 33,
        "blue": 34,
        "magenta": 35,
        "cyan": 36,
        "light_gray": 37,
        "default": 39,
        "dark_gray": 90,
        "light_red": 91,
        "light_green": 92,
        "light_yellow": 93,
        "light_blue": 94,
        "light_magenta": 95,
        "light_cyan": 96,
        "white": 97,
    }

    BACKGROUND_COLORS = {
        "black": 40,
        "red": 41,
        "green": 42,
        "yellow": 43,
        "blue": 44,
        "magenta": 45,
        "cyan": 46,
        "light_gray": 47,
        "default": 49,
        "dark_gray": 100,
        "light_red": 101,
        "light_green": 102,
        "light_yellow": 103,
        "light_blue": 104,
        "light_magenta": 105,
        "light_cyan": 106,
        "white": 107,
    }

    OPTIONS = {
        "bold": 1,
        "dark": 2,
        "italic": 3,
        "underline": 4,
        "blink": 5,
        "reverse": 7,
        "conceal": 8,
    }

    def __init__(self, foreground=None, background=None, options=None):
        self._fg = foreground
        self._bg = background
        self._foreground = None
        self._background = None

        if foreground:
            self.set_foreground(foreground)

        if background:
            self.set_background(background)

        options = options or []
        if not isinstance(options, list):
            options = [options]

        self.set_options(options)

    @property
    def foreground(self):
        return self._fg

    @property
    def background(self):
        return self._bg

    @property
    def options(self):
        return list(self._options.values())

    def set_foreground(self, foreground):
        if foreground not in self.FOREGROUND_COLORS:
            raise ValueError(
                'Invalid foreground specified: "{}". Expected one of ({})'.format(
                    foreground, ", ".join(self.FOREGROUND_COLORS.keys())
                )
            )

        self._foreground = self.FOREGROUND_COLORS[foreground]

    def set_background(self, background):
        if background not in self.FOREGROUND_COLORS:
            raise ValueError(
                'Invalid background specified: "{}". Expected one of ({})'.format(
                    background, ", ".join(self.BACKGROUND_COLORS.keys())
                )
            )

        self._background = self.BACKGROUND_COLORS[background]

    def set_option(self, option):
        if option not in self.OPTIONS:
            raise ValueError(
                'Invalid option specified: "{}". Expected one of ({})'.format(
                    option, ", ".join(self.OPTIONS.keys())
                )
            )

        if option not in self._options:
            self._options[self.OPTIONS[option]] = option

    def unset_option(self, option):
        if not option in self.OPTIONS:
            raise ValueError(
                'Invalid option specified: "{}". Expected one of ({})'.format(
                    option, ", ".join(self.OPTIONS.keys())
                )
            )

        del self._options[self.OPTIONS[option]]

    def set_options(self, options):
        self._options = OrderedDict()

        for option in options:
            self.set_option(option)

    def apply(self, text):
        codes = []

        if self._foreground:
            codes.append(self._foreground)

        if self._background:
            codes.append(self._background)

        if len(self._options):
            codes += list(self._options.keys())

        if not len(codes):
            return text

        return "\033[%sm%s\033[0m" % (";".join(map(str, codes)), text)

    def __eq__(self, other):
        return (
            other._foreground == self._foreground
            and other._background == self._background
            and other._options == self._options
        )



# -*- coding: utf-8 -*-

import pytest

from pastel.style import Style


def test_push(stack):
    s1 = Style("white", "black")
    s2 = Style("yellow", "blue")
    stack.push(s1)
    stack.push(s2)

    assert s2 == stack.get_current()

    s3 = Style("green", "red")
    stack.push(s3)

    assert s3 == stack.get_current()


def test_pop(stack):
    s1 = Style("white", "black")
    s2 = Style("yellow", "blue")
    stack.push(s1)
    stack.push(s2)

    assert s2 == stack.pop()
    assert s1 == stack.pop()


def test_pop_empty(stack):
    assert isinstance(stack.pop(), Style)


def test_pop_not_last(stack):
    s1 = Style("white", "black")
    s2 = Style("yellow", "blue")
    s3 = Style("green", "red")
    stack.push(s1)
    stack.push(s2)
    stack.push(s3)

    assert s2 == stack.pop(s2)
    assert s1 == stack.pop()


def test_invalid_pop(stack):
    s1 = Style("white", "black")
    s2 = Style("yellow", "blue")
    stack.push(s1)

    with pytest.raises(ValueError):
        stack.pop(s2)



# -*- coding: utf-8 -*-

import pytest

from pastel import Pastel
from pastel.style import Style
from pastel.stack import StyleStack


@pytest.fixture
def pastel():
    return Pastel(True)


@pytest.fixture
def non_decorated_pastel():
    return Pastel(False)


@pytest.fixture
def style():
    return Style()


@pytest.fixture
def stack():
    return StyleStack()



# -*- coding: utf-8 -*-

__version__ = "0.2.0"

from .pastel import Pastel


_PASTEL = Pastel(True)


def colorize(message):
    """
    Formats a message to a colorful string.

    :param message: The message to format.
    :type message: str

    :rtype: str
    """
    with _PASTEL.colorized():
        return _PASTEL.colorize(message)


def with_colors(colorized):
    """
    Enable or disable colors.

    :param decorated: Whether to active colors or not.
    :type decorated: bool

    :rtype: None
    """
    _PASTEL.with_colors(colorized)


def add_style(name, fg=None, bg=None, options=None):
    """
    Adds a new style.

    :param name: The name of the style
    :type name: str

    :param fg: The foreground color
    :type fg: str or None

    :param bg: The background color
    :type bg: str or None

    :param options: The style options
    :type options: list or str or None
    """
    _PASTEL.add_style(name, fg, bg, options)


def remove_style(name):
    """
    Removes a style.

    :param name: The name of the style to remove.
    :type name: str

    :rtype: None
    """
    _PASTEL.remove_style(name)


def pastel(colorized=True):
    """
    Returns a new Pastel instance.

    :rtype: Pastel
    """
    return Pastel(colorized)



# -*- coding: utf-8 -*-

import pytest

from pastel.style import Style


def test_init():
    style = Style("green", "black", ["bold", "underline"])
    assert "\033[32;40;1;4mfoo\033[0m" == style.apply("foo")
    assert "green" == style.foreground
    assert "black" == style.background
    assert ["bold", "underline"] == style.options

    style = Style("red", None, ["blink"])
    assert "\033[31;5mfoo\033[0m" == style.apply("foo")

    style = Style(None, "white")
    assert "\033[107mfoo\033[0m" == style.apply("foo")

    style = Style("red", None, "blink")
    assert "\033[31;5mfoo\033[0m" == style.apply("foo")


def test_foreground(style):
    style.set_foreground("black")
    assert "\033[30mfoo\033[0m" == style.apply("foo")

    style.set_foreground("blue")
    assert "\033[34mfoo\033[0m" == style.apply("foo")

    with pytest.raises(ValueError):
        style.set_foreground("undefined-color")


def test_background(style):
    style.set_background("black")
    assert "\033[40mfoo\033[0m" == style.apply("foo")

    style.set_background("yellow")
    assert "\033[43mfoo\033[0m" == style.apply("foo")

    with pytest.raises(ValueError):
        style.set_background("undefined-color")


def test_options(style):
    style.set_options(["reverse", "conceal"])
    assert "\033[7;8mfoo\033[0m" == style.apply("foo")

    style.set_option("bold")
    assert "\033[7;8;1mfoo\033[0m" == style.apply("foo")

    style.unset_option("reverse")
    assert "\033[8;1mfoo\033[0m" == style.apply("foo")

    style.set_option("bold")
    assert "\033[8;1mfoo\033[0m" == style.apply("foo")

    style.set_options(["bold"])
    assert "\033[1mfoo\033[0m" == style.apply("foo")

    with pytest.raises(ValueError) as e:
        style.set_option("foo")

    assert 'Invalid option specified: "foo"' in str(e.value)

    with pytest.raises(ValueError) as e:
        style.unset_option("foo")

    assert 'Invalid option specified: "foo"' in str(e.value)
